/*
 *  Routines for enforcing the constraint that the parent
 *  variables must form a directed acyclic graph.
 */
#ifndef _DAG_H_
#define _DAG_H_

class Instance;

class DAG
{
    public:
	DAG( Instance *instance );
	~DAG();

	/*
	 *  Propagate the acyclic constraint over the parent variables.
	 */
	bool propagate(
		int *currentParents, int *currentOrdering, int *currentDepth,
		int currentLevel, bool *instantiatedP, bool **checkingP, int v );

    private:
	Instance *instance;

	/*
	 *  Pruning rule based on necessary edges.
	 */
	bool propagateNecessaryEdges(
		int currentLevel, bool *instantiatedP, bool **checkingP, int v );

	/*
	 *  Satisfiability test.
	 */
	bool isAcyclic( int *currentOrdering, int currentLevel );
	bool isAcyclic( int *currentOrdering, int currentLevel, int v, int a );
	bool isAcyclic( int *currentOrdering, int *currentDepth, int currentLevel );
};

#endif // _DAG_H_
